"""Version."""
from __future__ import unicode_literals
from .__version__ import version, version_info

__version__ = version
__version_info__ = version_info
__app__ = "SubClrSchm"
__status__ = version_info[3]
__maintainers__ = [("Isaac Muse", "IsaacMuse@gmail.com")]
__help__ = "https://github.com/facelessuser/subclrschm/issues"
__manual__ = "https://github.com/facelessuser/subclrschm/blob/master/README.md"
