"""Default theme."""
theme = {
    "name": "Grayscale",
    "settings": [
        {
            "settings": {
                "activeGuide": "#888888",
                "background": "#FFFFFF",
                "bracketContentsForeground": "#000000",
                "bracketContentsOptions": "underline",
                "bracketsForeground": "#000000",
                "bracketsOptions": "underline",
                "caret": "#000000",
                "findHighlight": "#666666",
                "findHighlightForeground": "#000000",
                "foreground": "#000000",
                "gutter": "#E5E5E5",
                "gutterForeground": "#000000",
                "inactiveSelection": "#888888",
                "invisibles": "#323232",
                "lineHighlight": "#2E2E2E22",
                "selection": "#666666",
                "selectionForeground": "#FFFFFF",
                "tagsOptions": "stippled_underline"
            }
        },
        {
            "name": "Comment",
            "scope": "comment",
            "settings": {
                "foreground": "#A5A5A5"
            }
        },
        {
            "name": "Storage type",
            "scope": "storage.type",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        },
        {
            "name": "Class name",
            "scope": "entity.name.class",
            "settings": {
                "fontStyle": "underline",
                "foreground": "#000000"
            }
        },
        {
            "name": "Inherited class",
            "scope": "entity.other.inherited-class",
            "settings": {
                "fontStyle": "italic underline",
                "foreground": "#000000"
            }
        },
        {
            "name": "Function argument",
            "scope": "variable.parameter",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        },
        {
            "name": "Library class/type",
            "scope": "support.type, support.class",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        }
    ],
    "uuid": "07379361-ce49-4e6c-a03f-26378a7a2131"
}
